local function debugWorldDraw(world)
   local bodies = world:getBodyList()
   
   for b=#bodies,1,-1 do
      local body = bodies[b]
      local bx,by = body:getPosition()
      local bodyAngle = body:getAngle()
      love.graphics.push()
      love.graphics.translate(bx,by)
      love.graphics.rotate(bodyAngle)
      
      math.randomseed(1) --for color generation
      
      local fixtures = body:getFixtureList()
      for i=1,#fixtures do
         local fixture = fixtures[i]
         local shape = fixture:getShape()
         local shapeType = shape:getType()
         local isSensor = fixture:isSensor()
         
         if (isSensor) then
            love.graphics.setColor(0,0,255,96)
         else
            love.graphics.setColor(math.random(32,200),math.random(32,200),math.random(32,200),96)
         end
         
         love.graphics.setLineWidth(1)
         if (shapeType == "circle") then
            local x,y = fixture:getMassData() --0.9.0 missing circleshape:getPoint()
            --local x,y = shape:getPoint() --0.9.1
            local radius = shape:getRadius()
            love.graphics.circle("fill",x,y,radius,15)
            love.graphics.setColor(0,0,0,255)
            love.graphics.circle("line",x,y,radius,15)
            local eyeRadius = radius/4
            love.graphics.setColor(0,0,0,255)
            love.graphics.circle("fill",x+radius-eyeRadius,y+0,eyeRadius,10)
         elseif (shapeType == "polygon") then
            local points = {shape:getPoints()}
            love.graphics.polygon("fill",points)
            love.graphics.setColor(0,0,0,255)
            love.graphics.polygon("line",points)
         elseif (shapeType == "edge") then
            love.graphics.setColor(0,0,0,255)
            love.graphics.line(shape:getPoints())
         elseif (shapeType == "chain") then
            love.graphics.setColor(0,0,0,255)
            love.graphics.line(shape:getPoints())
         end
      end
      love.graphics.pop()
   end
   
   local joints = world:getJointList()
   for index,joint in pairs(joints) do
      love.graphics.setColor(0,255,0,255)
      local x1,y1,x2,y2 = joint:getAnchors()
      if (x1 and x2) then
         love.graphics.setLineWidth(3)
         love.graphics.line(x1,y1,x2,y2)
      else
         love.graphics.setPointSize(3)
         if (x1) then
            love.graphics.point(x1,y1)
         end
         if (x2) then
            love.graphics.point(x2,y2)
         end
      end
   end
   
   local contacts = world:getContactList()
   for i=1,#contacts do
      love.graphics.setColor(255,0,0,255)
      love.graphics.setPointSize(3)
      local x1,y1,x2,y2 = contacts[i]:getPositions()
      if (x1) then
         love.graphics.point(x1,y1)
      end
      if (x2) then
         love.graphics.point(x2,y2)
      end
   end
end

lg = love.graphics
lm = love.mouse
lt = love.timer
lk = love.keyboard
lp = love.physics


-- empty tables
FLOORS = {} -- bottom-most
CEILINGS = {} -- upper-most
WALLS = {} -- left/right
--
INSTANCES = {}
--
RECTANGLES = {}
TRIANGLES = {}
CIRCLES = {}
CHAINS = {}


require("classes/rectangle")
require("classes/triangle")
require("classes/circle")


love.load = function()
	lg.setBackgroundColor(48, 48, 48, 255)
	lg.setFont(lg.newFont(9))
	--lm.setVisible(false)
	
	-- physics world
	x_grav, y_grav = 0, 100
	can_world_sleep = true
	world = lp.newWorld(x_grav, y_grav, can_world_sleep)
	
	room = "test1"
	level = nil
	maps = {
		start = {
			{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
			{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
		}
	}
end


love.update = function(dt)
	if(room == "test1") then
		level = maps.start
	end
	
	world:update(dt)
	
	if(not level_created) then
		local y, x
		for y = 1, #level do
			for x = 1, #level[y] do
				if(level[y][x] == 1) then
					static = rectangle.new("static", (x * 32) + 16, y * 32, 32, 32)
					table.insert(RECTANGLES, static)
				elseif(level[y][x] == 2) then
					slope_bl = triangle.new("static", "bl", (x * 16) + 16, (y * 16) + 16, 32, 32)
					table.insert(TRIANGLES, slope_bl)
				elseif(level[y][x] == 3) then
					slope_br = triangle.new("static", "br", x * 32, y * 32, 32, 32)
					table.insert(TRIANGLES, slope_br)
				elseif(level[y][x] == 4) then
					slope_tl = triangle.new("static", "tl", (x * 16) + 16, y * 16, 32, 32)
					table.insert(TRIANGLES, slope_tl)
				elseif(level[y][x] == 5) then
					slope_tr = triangle.new("static", "tr", x * 32, y * 32, 32, 32)
					table.insert(TRIANGLES, slope_tr)
				end
				level_created = true
			end
		end
	end
end


local scale_amnt = 1
love.draw = function()
	debugWorldDraw(world)
	do return end

	lg.setLineStyle("rough")
	
	
	lg.push() -- everything after will be translated...
	lg.scale(scale_amnt, scale_amnt)
	
	-- camera
	--lg.translate(lm.getX() - lg.getWidth() / 2, lm.getY() - lg.getHeight() / 2)
	
	local grid_size = 16
	local tile_x = math.floor(lm.getX() / grid_size)
   local tile_y = math.floor(lm.getY() / grid_size)
	
	local i, j
	for i = 1, math.ceil(lg.getWidth() / grid_size) do
		for j = 1, math.ceil(lg.getHeight() / grid_size) do
			lg.setColor(255, 255, 255, 32)
			lg.rectangle("line", (i * grid_size) - grid_size, (j * grid_size) - grid_size, grid_size, grid_size)
      end
   end
	
	local i, j
	for i, j in ipairs(RECTANGLES) do
		j:draw()
	end
	
	local i, j
	for i, j in ipairs(TRIANGLES) do
		j:draw()
	end
	
	local i, j
	for i, j in ipairs(CIRCLES) do
		j:draw()
	end
	
	lg.pop() -- no translation afterwards...
	
	lg.setColor(255, 0, 0, 255)
	lg.print(
	"fps: " .. lt.getFPS() ..
	"\nscale_amount: " .. scale_amnt ..
	"\n# of rectangles: " .. #RECTANGLES, 4, 4)
	
	lg.setColor(255, 255, 255, 255)
	lg.circle("fill", lm.getX(), lm.getY(), 4, 4)
end


love.mousepressed = function(x, y, b)
	if(b == "wd") then
		if(scale_amnt ~= 1) then scale_amnt = scale_amnt - 0.25 end
	elseif(b == "wu") then
		if(scale_amnt ~= 4.25) then scale_amnt = scale_amnt + 0.25 end
	elseif(b == "l") then
		table.insert(CIRCLES, circle.new("dynamic", x, y, 32))
	end
end